@@ -597,21 +597,23 @@ static int64_t export_data(
 }
 
 void set_ff_bits(md_t *st, int32_t fc){
-  if (fc == 0) {
-    st->ff_bits = 0;
-  } else {
-    if (fc < pow(2,8)) {
-      st->ff_bits = 8;
+    if (fc == 0) {
+        st->ff_bits = 0;
     } else {
-      if (fc < pow(2,16)) {
-        st->ff_bits = 16;
-      } else {
-        if (fc < pow(2,32)) {
-          st->ff_bits = 32;
+        if (fc < (int32_t)(1) << 8) {
+            st->ff_bits = 8;
+        } else {
+            if (fc < (int32_t)(1) << 16) {
+                st->ff_bits = 16;
+            } else {
+                if (fc < (int32_t)(1) << 23) {
+                    st->ff_bits = 32;
+                } else {
+                    st->ff_bits = 32;
+                }
+            }
         }
-      }
     }
-  }
 }
 
 /* return 1 if validation was possible, zero otherwise */
@@ -983,7 +985,7 @@ void set_function_pointers(
       sba_reduce_dense_row_by_known_pivots_sparse_ff_32 =
         sba_reduce_dense_row_by_known_pivots_sparse_31_bit;
       /* if coeffs are smaller than 17 bit we can optimize reductions */
-      if (st->fc < pow(2, 18)) {
+      if (st->fc < (int32_t)(1) << 18) {
         reduce_dense_row_by_all_pivots_ff_32 =
           reduce_dense_row_by_all_pivots_17_bit;
         reduce_dense_row_by_old_pivots_ff_32 =
@@ -993,7 +995,6 @@ void set_function_pointers(
         reduce_dense_row_by_dense_new_pivots_ff_32  =
           reduce_dense_row_by_dense_new_pivots_17_bit;
       } else {
-        if (st->fc < pow(2, 31)) {
           reduce_dense_row_by_all_pivots_ff_32 =
             reduce_dense_row_by_all_pivots_31_bit;
           reduce_dense_row_by_old_pivots_ff_32 =
@@ -1002,16 +1003,6 @@ void set_function_pointers(
             reduce_dense_row_by_known_pivots_sparse_31_bit;
           reduce_dense_row_by_dense_new_pivots_ff_32  =
             reduce_dense_row_by_dense_new_pivots_31_bit;
-        } else {
-          reduce_dense_row_by_all_pivots_ff_32 =
-            reduce_dense_row_by_all_pivots_31_bit;
-          reduce_dense_row_by_old_pivots_ff_32 =
-            reduce_dense_row_by_old_pivots_31_bit;
-          reduce_dense_row_by_known_pivots_sparse_ff_32 =
-            reduce_dense_row_by_known_pivots_sparse_32_bit;
-          reduce_dense_row_by_dense_new_pivots_ff_32  =
-            reduce_dense_row_by_dense_new_pivots_31_bit;
-        }
       }
       break;
 
@@ -1040,7 +1031,7 @@ void set_function_pointers(
       normalize_initial_basis = normalize_initial_basis_ff_32;
 
       /* if coeffs are smaller than 17 bit we can optimize reductions */
-      if (st->fc < pow(2, 18)) {
+      if (st->fc < (int32_t)(1) << 18) {
         reduce_dense_row_by_all_pivots_ff_32 =
           reduce_dense_row_by_all_pivots_17_bit;
         reduce_dense_row_by_old_pivots_ff_32 =
@@ -1050,7 +1041,6 @@ void set_function_pointers(
         reduce_dense_row_by_dense_new_pivots_ff_32  =
           reduce_dense_row_by_dense_new_pivots_17_bit;
       } else {
-        if (st->fc < pow(2, 31)) {
           reduce_dense_row_by_all_pivots_ff_32 =
             reduce_dense_row_by_all_pivots_31_bit;
           reduce_dense_row_by_old_pivots_ff_32 =
@@ -1059,16 +1049,6 @@ void set_function_pointers(
             reduce_dense_row_by_known_pivots_sparse_31_bit;
           reduce_dense_row_by_dense_new_pivots_ff_32  =
             reduce_dense_row_by_dense_new_pivots_31_bit;
-        } else {
-          reduce_dense_row_by_all_pivots_ff_32 =
-            reduce_dense_row_by_all_pivots_31_bit;
-          reduce_dense_row_by_old_pivots_ff_32 =
-            reduce_dense_row_by_old_pivots_31_bit;
-          reduce_dense_row_by_known_pivots_sparse_ff_32 =
-            reduce_dense_row_by_known_pivots_sparse_32_bit;
-          reduce_dense_row_by_dense_new_pivots_ff_32  =
-            reduce_dense_row_by_dense_new_pivots_31_bit;
-        }
       }
   }
 }
@@ -1117,7 +1097,7 @@ static inline void reset_function_pointers(
         const uint32_t laopt
         )
 {
-    if (prime < pow(2,8)) {
+    if (prime < (int32_t)(1) << 8) {
         exact_linear_algebra    = exact_sparse_linear_algebra_ff_8;
         interreduce_matrix_rows = interreduce_matrix_rows_ff_8;
         normalize_initial_basis = normalize_initial_basis_ff_8;
@@ -1141,7 +1121,7 @@ static inline void reset_function_pointers(
             linear_algebra  = exact_sparse_linear_algebra_ff_8;
         }
     } else {
-        if (prime < pow(2,16)) {
+        if (prime < (int32_t)(1) << 16) {
             exact_linear_algebra    = exact_sparse_linear_algebra_ff_16;
             interreduce_matrix_rows = interreduce_matrix_rows_ff_16;
             normalize_initial_basis = normalize_initial_basis_ff_16;
@@ -1187,7 +1167,7 @@ static inline void reset_function_pointers(
               default:
                 linear_algebra  = exact_sparse_linear_algebra_ff_32;
             }
-            if (prime < pow(2,18)) {
+            if (prime < (int32_t)(1) << 18) {
                 reduce_dense_row_by_all_pivots_ff_32 =
                     reduce_dense_row_by_all_pivots_17_bit;
                 reduce_dense_row_by_old_pivots_ff_32 =
@@ -1197,7 +1177,6 @@ static inline void reset_function_pointers(
                 reduce_dense_row_by_dense_new_pivots_ff_32  =
                     reduce_dense_row_by_dense_new_pivots_17_bit;
             } else {
-              if (prime < pow(2,31)) {
                 reduce_dense_row_by_all_pivots_ff_32 =
                   reduce_dense_row_by_all_pivots_31_bit;
                 reduce_dense_row_by_old_pivots_ff_32 =
@@ -1206,16 +1185,6 @@ static inline void reset_function_pointers(
                   reduce_dense_row_by_known_pivots_sparse_31_bit;
                 reduce_dense_row_by_dense_new_pivots_ff_32  =
                   reduce_dense_row_by_dense_new_pivots_31_bit;
-              } else {
-                reduce_dense_row_by_all_pivots_ff_32 =
-                  reduce_dense_row_by_all_pivots_31_bit;
-                reduce_dense_row_by_old_pivots_ff_32 =
-                  reduce_dense_row_by_old_pivots_31_bit;
-                reduce_dense_row_by_known_pivots_sparse_ff_32 =
-                  reduce_dense_row_by_known_pivots_sparse_32_bit;
-                reduce_dense_row_by_dense_new_pivots_ff_32  =
-                  reduce_dense_row_by_dense_new_pivots_31_bit;
-              }
             }
         }
     }
@@ -1225,14 +1194,14 @@ static inline void reset_trace_function_pointers(
         const uint32_t prime
         )
 {
-    if (prime < pow(2,8)) {
+    if (prime < (int32_t)(1) << 8) {
         exact_linear_algebra        = exact_sparse_linear_algebra_ff_8;
         interreduce_matrix_rows     = interreduce_matrix_rows_ff_8;
         normalize_initial_basis     = normalize_initial_basis_ff_8;
         application_linear_algebra  = exact_application_sparse_linear_algebra_ff_8;
         trace_linear_algebra        = exact_trace_sparse_linear_algebra_ff_8;
     } else {
-        if (prime < pow(2,16)) {
+        if (prime < (int32_t)(1) << 16) {
             exact_linear_algebra        = exact_sparse_linear_algebra_ff_16;
             interreduce_matrix_rows     = interreduce_matrix_rows_ff_16;
             normalize_initial_basis     = normalize_initial_basis_ff_16;
@@ -1244,7 +1213,7 @@ static inline void reset_trace_function_pointers(
             normalize_initial_basis     = normalize_initial_basis_ff_32;
             application_linear_algebra  = exact_application_sparse_linear_algebra_ff_32;
             trace_linear_algebra        = exact_trace_sparse_linear_algebra_ff_32;
-            if (prime < pow(2,18)) {
+            if (prime < (int32_t)(1) << 18) {
                 reduce_dense_row_by_all_pivots_ff_32 =
                     reduce_dense_row_by_all_pivots_17_bit;
                 reduce_dense_row_by_old_pivots_ff_32 =
@@ -1256,7 +1225,6 @@ static inline void reset_trace_function_pointers(
                 reduce_dense_row_by_dense_new_pivots_ff_32  =
                     reduce_dense_row_by_dense_new_pivots_17_bit;
             } else {
-              if (prime < pow(2,31)) {
                 reduce_dense_row_by_all_pivots_ff_32 =
                   reduce_dense_row_by_all_pivots_31_bit;
                 reduce_dense_row_by_old_pivots_ff_32 =
@@ -1267,18 +1235,6 @@ static inline void reset_trace_function_pointers(
                   reduce_dense_row_by_known_pivots_sparse_31_bit;
                 reduce_dense_row_by_dense_new_pivots_ff_32  =
                   reduce_dense_row_by_dense_new_pivots_31_bit;
-              } else {
-                reduce_dense_row_by_all_pivots_ff_32 =
-                  reduce_dense_row_by_all_pivots_31_bit;
-                reduce_dense_row_by_old_pivots_ff_32 =
-                  reduce_dense_row_by_old_pivots_31_bit;
-                trace_reduce_dense_row_by_known_pivots_sparse_ff_32 =
-                  trace_reduce_dense_row_by_known_pivots_sparse_32_bit;
-                reduce_dense_row_by_known_pivots_sparse_ff_32 =
-                  reduce_dense_row_by_known_pivots_sparse_32_bit;
-                reduce_dense_row_by_dense_new_pivots_ff_32  =
-                  reduce_dense_row_by_dense_new_pivots_31_bit;
-              }
             }
         }
     }
